package MT::Plugin::SKR::LogRotater;
# LogRotater - Rotate log with the settings of each blogs.
#       Copyright (c) 2008 SKYARC System Co.,Ltd.
#       http://www.skyarc.co.jp/

use strict;
use MT;
use MT::Blog;
use MT::Log;
use Data::Dumper;#DEBUG

use vars qw( $MYNAME $VERSION );
$MYNAME = 'LogRotater';
$VERSION = '0.01';

use base qw( MT::Plugin );
my $plugin = __PACKAGE__->new({
        name => $MYNAME,
        id => lc $MYNAME,
        key => lc $MYNAME,
        version => $VERSION,
        description => <<HTMLHEREDOC,
Rotate log with the settings of each blogs.
HTMLHEREDOC
        config_template => 'config.tmpl',
        settings => new MT::PluginSettings([
            [ 'number_of_log',  { Default => 0 } ],     # Number of Logs
            [ 'unit_of_number', { Default => 0 } ],     # Unit of Number
            [ 'write_to_csv',   { Default => 0 } ],     # Stored to CSV
            [ 'path_to_csv',    { Default => undef } ], # Start path to CSV with
        ]),
});
MT->add_plugin( $plugin );

sub instance { $plugin; }

### Registry
sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        tasks => {
            $MYNAME => {
                name        => $MYNAME,
                frequency   => 60 * 60,# seconds
                code        => \&_hdlr_log_rotater,
            },
        },
    });
}



### Task to rotate logs
sub _hdlr_log_rotater {
    my $ct = time;

    # Enumerate all blogs
    my @blog_ids = ( 0 );# System logs
    my $blog_iter = MT::Blog->load_iter();
    while (my $blog = $blog_iter->()) {
        push @blog_ids, $blog->id;
    }

    # Do in each blogs
    foreach my $blog_id (@blog_ids) {
        my $scope = $blog_id ? "blog:$blog_id" : "system";
        my $number_of_log = &instance->get_config_value ('number_of_log', $scope)
            or next;# No settings or 0
        $number_of_log =~ s!\D!!g;
        ($number_of_log) = $number_of_log =~ m/^(\d+)$/
            or next;# may be 0

        # Other parameters
        my $unit_of_number = &instance->get_config_value ('unit_of_number', $scope) || 0;
        my $write_to_csv = &instance->get_config_value ('write_to_csv', $scope) || 0;
        my $path_to_csv = &instance->get_config_value ('path_to_csv', $scope) || undef;

        # Retrieve the logs to remove
        my @logs_remove = ();
        my $logs_iter = MT::Log->load_iter(
            { blog_id => $blog_id, class => '*' },# must be needed 'class'
            { sort => 'created_on', direction => 'ascend' });

        # Number of Days
        if ($unit_of_number) {
            my @ct = localtime( $ct - $number_of_log * 24 * 60 * 60 );
            my $ts = sprintf '%04d%02d%02d%02d%02d%02d', $ct[5]+1900, $ct[4]+1, @ct[3,2,1,0];
            while (my $log = $logs_iter->()) {
                $ts < $log->created_on
                    or push @logs_remove, $log->id;
            }
        }
        # Number of Logs
        else {
            while (my $log = $logs_iter->()) {
                0 < $number_of_log--
                    or push @logs_remove, $log->id;
            }
        }

        # Remove logs and write into CSV
        foreach my $log_id (@logs_remove) {
            my $log = MT::Log->load({ id => $log_id })
                or next;
            if (my $fh = open_csv_file ($write_to_csv, $path_to_csv, $log->created_on)) {
                write_log_to_csv_file ($fh, $log);
                close $fh;
            }
            $log->remove;# bye-bye
        }
    }
}



### Columns to write to CSV
my @log_columns = qw{ category class created_on ip level message };

### Open CSV file and return file handle
sub open_csv_file {
    my ($write_to_csv, $path_to_csv, $created_on) = @_;

    defined $path_to_csv && $path_to_csv ne ''
        or return undef;

    if ($write_to_csv == 1) {# Daily
        $path_to_csv .= substr($created_on, 0, 8). '.csv';
    }
    elsif ($write_to_csv == 2) {# Weekly
        return undef;# not implemented
    }
    elsif ($write_to_csv == 3) {# Monthly
        $path_to_csv .= substr($created_on, 0, 6). '.csv';
    }
    elsif ($write_to_csv == 4) {# Yearly
        $path_to_csv .= substr($created_on, 0, 4). '.csv';
    }
    else {
        return undef;
    }

    #
    if (open (my $fh, ">>$path_to_csv")) {
        print $fh join( ',', @log_columns ). "\n" if -z $path_to_csv;
        return $fh;
    }
    return undef;
}

### Write a log into a line.
sub write_log_to_csv_file {
    my ($fh, $log) = @_;
    print $fh join( ',', map { $log->$_ } @log_columns ). "\n";
}

1;
__END__
